@@ -25,10 +25,13 @@ namespace WASI {
 
 namespace {
 
+inline void WASMEDGE_WINAPI_WINAPI_CC
+EmptyOverlappedCompletionRoutine(DWORD_, DWORD_, LPOVERLAPPED_) noexcept {}
+
 #if WINAPI_PARTITION_DESKTOP
 inline constexpr uint64_t combineHighLow(uint32_t HighPart,
                                          uint32_t LowPart) noexcept {
-  const ULARGE_INTEGER_ Temp = {.LowPart = LowPart, .HighPart = HighPart};
+  const ULARGE_INTEGER_ Temp = {/* LowPart */ LowPart, /* HighPart */ HighPart};
   return Temp.QuadPart;
 }
 #endif
@@ -86,6 +89,10 @@ WasiExpect<std::tuple<DWORD_, DWORD_, DWORD_>> inline constexpr getOpenFlags(
     AttributeFlags |= FILE_FLAG_WRITE_THROUGH_;
     FdFlags &= ~__WASI_FDFLAGS_DSYNC;
   }
+  if (FdFlags & __WASI_FDFLAGS_NONBLOCK) {
+    // Ignore NONBLOCK flag
+    FdFlags &= ~__WASI_FDFLAGS_NONBLOCK;
+  }
   if (OpenFlags & __WASI_OFLAGS_DIRECTORY) {
     AttributeFlags |= FILE_ATTRIBUTE_DIRECTORY_;
     OpenFlags &= ~__WASI_OFLAGS_DIRECTORY;
@@ -222,9 +229,9 @@ getHandlePath(HANDLE_ Handle) noexcept {
   // First get the path of the handle
 #if NTDDI_VERSION >= NTDDI_VISTA
   std::array<wchar_t, UNICODE_STRING_MAX_CHARS_ + 1> Buffer;
-  const auto Size =
-      GetFinalPathNameByHandleW(Handle, Buffer.data(), Buffer.size(),
-                                FILE_NAME_NORMALIZED_ | VOLUME_NAME_DOS_);
+  const auto Size = GetFinalPathNameByHandleW(
+      Handle, Buffer.data(), static_cast<DWORD_>(Buffer.size()),
+      FILE_NAME_NORMALIZED_ | VOLUME_NAME_DOS_);
   if (unlikely(Size == 0)) {
     return WasiUnexpect(detail::fromLastError(GetLastError()));
   }
@@ -508,14 +515,6 @@ WasiExpect<void> FindHolderBase<T>::emplace(HANDLE_ PathHandle) noexcept {
   return {};
 }
 
-template <typename T> void FindHolderBase<T>::reset() noexcept {
-  Proxy::doReset(static_cast<T &>(*this));
-  Path.clear();
-  Handle = nullptr;
-  Cookie = 0;
-  Buffer.clear();
-}
-
 template <typename T>
 WasiExpect<void> FindHolderBase<T>::seek(uint64_t NewCookie) noexcept {
   if (NewCookie < Cookie) {
@@ -768,15 +767,18 @@ WasiExpect<void> INode::fdAdvise(__wasi_filesize_t Offset, __wasi_filesize_t,
 
 WasiExpect<void> INode::fdAllocate(__wasi_filesize_t Offset,
                                    __wasi_filesize_t Len) const noexcept {
-  if (unlikely(Offset > std::numeric_limits<int64_t>::max())) {
+  if (unlikely(Offset >
+               static_cast<uint64_t>(std::numeric_limits<int64_t>::max()))) {
     return WasiUnexpect(__WASI_ERRNO_INVAL);
   }
 
-  if (unlikely(Len > std::numeric_limits<int64_t>::max())) {
+  if (unlikely(Len >
+               static_cast<uint64_t>((std::numeric_limits<int64_t>::max())))) {
     return WasiUnexpect(__WASI_ERRNO_INVAL);
   }
 
-  if (unlikely((Offset + Len) > std::numeric_limits<int64_t>::max())) {
+  if (unlikely((Offset + Len) >
+               static_cast<uint64_t>(std::numeric_limits<int64_t>::max()))) {
     return WasiUnexpect(__WASI_ERRNO_INVAL);
   }
 
@@ -798,12 +800,12 @@ WasiExpect<void> INode::fdAllocate(__wasi_filesize_t Offset,
     return WasiUnexpect(detail::fromLastError(GetLastError()));
   }
 #else
-  LARGE_INTEGER_ Old = {.QuadPart = 0};
+  LARGE_INTEGER_ Old = _LARGE_INTEGER(0);
   if (unlikely(!SetFilePointerEx(Handle, Old, &Old, FILE_CURRENT_))) {
     return WasiUnexpect(detail::fromLastError(GetLastError()));
   }
 
-  LARGE_INTEGER_ New = {.QuadPart = RequestSize};
+  LARGE_INTEGER_ New = _LARGE_INTEGER(RequestSize);
   if (unlikely(!SetFilePointerEx(Handle, New, nullptr, FILE_BEGIN_))) {
     return WasiUnexpect(detail::fromLastError(GetLastError()));
   }
@@ -875,7 +877,8 @@ INode::fdFilestatGet(__wasi_filestat_t &FileStat) const noexcept {
 
 WasiExpect<void>
 INode::fdFilestatSetSize(__wasi_filesize_t Size) const noexcept {
-  if (unlikely(Size > std::numeric_limits<int64_t>::max())) {
+  if (unlikely(Size >
+               static_cast<uint64_t>(std::numeric_limits<int64_t>::max()))) {
     return WasiUnexpect(__WASI_ERRNO_INVAL);
   }
 
@@ -890,12 +893,12 @@ INode::fdFilestatSetSize(__wasi_filesize_t Size) const noexcept {
     return WasiUnexpect(detail::fromLastError(GetLastError()));
   }
 #else
-  LARGE_INTEGER_ Old = {.QuadPart = 0};
+  LARGE_INTEGER_ Old = _LARGE_INTEGER(0);
   if (unlikely(!SetFilePointerEx(Handle, Old, &Old, FILE_CURRENT_))) {
     return WasiUnexpect(detail::fromLastError(GetLastError()));
   }
 
-  LARGE_INTEGER_ New = {.QuadPart = RequestSize};
+  LARGE_INTEGER_ New = _LARGE_INTEGER(RequestSize);
   if (unlikely(!SetFilePointerEx(Handle, New, nullptr, FILE_BEGIN_))) {
     return WasiUnexpect(detail::fromLastError(GetLastError()));
   }
@@ -951,7 +954,7 @@ WasiExpect<void> INode::fdPread(Span<Span<uint8_t>> IOVs,
                                 __wasi_size_t &NRead) const noexcept {
   WasiExpect<void> Result;
   std::vector<OVERLAPPED_> Queries(IOVs.size());
-  ULARGE_INTEGER_ LocalOffset = {.QuadPart = Offset};
+  ULARGE_INTEGER_ LocalOffset = _ULARGE_INTEGER(Offset);
 
   for (size_t I = 0; I < IOVs.size(); ++I) {
     auto &IOV = IOVs[I];
@@ -960,7 +963,7 @@ WasiExpect<void> INode::fdPread(Span<Span<uint8_t>> IOVs,
     Query.OffsetHigh = LocalOffset.HighPart;
     Query.hEvent = nullptr;
     if (!ReadFileEx(Handle, IOV.data(), static_cast<uint32_t>(IOV.size()),
-                    &Query, nullptr)) {
+                    &Query, &EmptyOverlappedCompletionRoutine)) {
       if (unlikely(GetLastError() != ERROR_IO_PENDING_)) {
         Result = WasiUnexpect(detail::fromLastError(GetLastError()));
         Queries.resize(I);
@@ -995,7 +998,7 @@ WasiExpect<void> INode::fdPwrite(Span<Span<const uint8_t>> IOVs,
   const bool Append = SavedFdFlags & __WASI_FDFLAGS_APPEND;
   WasiExpect<void> Result;
   std::vector<OVERLAPPED_> Queries(IOVs.size());
-  ULARGE_INTEGER_ LocalOffset = {.QuadPart = Offset};
+  ULARGE_INTEGER_ LocalOffset = _ULARGE_INTEGER(Offset);
 
   for (size_t I = 0; I < IOVs.size(); ++I) {
     auto &IOV = IOVs[I];
@@ -1009,7 +1012,7 @@ WasiExpect<void> INode::fdPwrite(Span<Span<const uint8_t>> IOVs,
     }
     Query.hEvent = nullptr;
     if (!WriteFileEx(Handle, IOV.data(), static_cast<uint32_t>(IOV.size()),
-                     &Query, nullptr)) {
+                     &Query, &EmptyOverlappedCompletionRoutine)) {
       if (const auto Error = GetLastError();
           unlikely(Error != ERROR_IO_PENDING_ && Error != ERROR_HANDLE_EOF_)) {
         Result = WasiUnexpect(detail::fromLastError(Error));
@@ -1025,9 +1028,9 @@ WasiExpect<void> INode::fdPwrite(Span<Span<const uint8_t>> IOVs,
   NWritten = 0;
   for (size_t I = 0; I < Queries.size(); ++I) {
     auto &Query = Queries[I];
-    DWORD_ NumberOfBytesRead = 0;
+    DWORD_ NumberOfBytesWrite = 0;
     if (unlikely(
-            !GetOverlappedResult(Handle, &Query, &NumberOfBytesRead, true))) {
+            !GetOverlappedResult(Handle, &Query, &NumberOfBytesWrite, true))) {
       Result = WasiUnexpect(detail::fromLastError(GetLastError()));
       CancelIo(Handle);
       for (size_t J = I + 1; J < Queries.size(); ++J) {
@@ -1035,7 +1038,7 @@ WasiExpect<void> INode::fdPwrite(Span<Span<const uint8_t>> IOVs,
       }
       break;
     }
-    NWritten += NumberOfBytesRead;
+    NWritten += NumberOfBytesWrite;
   }
 
   return Result;
@@ -1045,7 +1048,7 @@ WasiExpect<void> INode::fdRead(Span<Span<uint8_t>> IOVs,
                                __wasi_size_t &NRead) const noexcept {
   WasiExpect<void> Result;
   std::vector<OVERLAPPED_> Queries(IOVs.size());
-  LARGE_INTEGER_ OldOffset = {.QuadPart = 0};
+  LARGE_INTEGER_ OldOffset = _LARGE_INTEGER(0);
   if (unlikely(
           !SetFilePointerEx(Handle, OldOffset, &OldOffset, FILE_CURRENT_))) {
     return WasiUnexpect(detail::fromLastError(GetLastError()));
@@ -1059,7 +1062,7 @@ WasiExpect<void> INode::fdRead(Span<Span<uint8_t>> IOVs,
     Query.OffsetHigh = static_cast<DWORD_>(LocalOffset.HighPart);
     Query.hEvent = nullptr;
     if (!ReadFileEx(Handle, IOV.data(), static_cast<uint32_t>(IOV.size()),
-                    &Query, nullptr)) {
+                    &Query, &EmptyOverlappedCompletionRoutine)) {
       if (unlikely(GetLastError() != ERROR_IO_PENDING_)) {
         Result = WasiUnexpect(detail::fromLastError(GetLastError()));
         Queries.resize(I);
@@ -1113,7 +1116,7 @@ WasiExpect<void> INode::fdReaddir(Span<uint8_t> Buffer,
   do {
     const auto Written = Find.write(Buffer);
     Buffer = Buffer.subspan(Written);
-    Size += Written;
+    Size += static_cast<uint32_t>(Written);
     if (unlikely(Buffer.empty())) {
       break;
     }
@@ -1141,7 +1144,7 @@ WasiExpect<void> INode::fdSeek(__wasi_filedelta_t Offset,
                                __wasi_whence_t Whence,
                                __wasi_filesize_t &Size) const noexcept {
   DWORD_ SysWhence = toWhence(Whence);
-  LARGE_INTEGER_ Pointer = {.QuadPart = Offset};
+  LARGE_INTEGER_ Pointer = _LARGE_INTEGER(Offset);
   if (unlikely(!SetFilePointerEx(Handle, Pointer, &Pointer, SysWhence))) {
     return WasiUnexpect(detail::fromLastError(GetLastError()));
   }
@@ -1157,7 +1160,7 @@ WasiExpect<void> INode::fdSync() const noexcept {
 }
 
 WasiExpect<void> INode::fdTell(__wasi_filesize_t &Size) const noexcept {
-  LARGE_INTEGER_ Pointer = {.QuadPart = 0};
+  LARGE_INTEGER_ Pointer = _LARGE_INTEGER(0);
   if (unlikely(!SetFilePointerEx(Handle, Pointer, &Pointer, FILE_CURRENT_))) {
     return WasiUnexpect(detail::fromLastError(GetLastError()));
   }
@@ -1170,7 +1173,7 @@ WasiExpect<void> INode::fdWrite(Span<Span<const uint8_t>> IOVs,
   const bool Append = SavedFdFlags & __WASI_FDFLAGS_APPEND;
   WasiExpect<void> Result;
   std::vector<OVERLAPPED_> Queries(IOVs.size());
-  LARGE_INTEGER_ OldOffset = {.QuadPart = 0};
+  LARGE_INTEGER_ OldOffset = _LARGE_INTEGER(0);
   if (!Append) {
     if (unlikely(
             !SetFilePointerEx(Handle, OldOffset, &OldOffset, FILE_CURRENT_))) {
@@ -1191,7 +1194,7 @@ WasiExpect<void> INode::fdWrite(Span<Span<const uint8_t>> IOVs,
     }
     Query.hEvent = nullptr;
     if (!WriteFileEx(Handle, IOV.data(), static_cast<uint32_t>(IOV.size()),
-                     &Query, nullptr)) {
+                     &Query, &EmptyOverlappedCompletionRoutine)) {
       if (const auto Error = GetLastError();
           unlikely(Error != ERROR_IO_PENDING_ && Error != ERROR_HANDLE_EOF_)) {
         Result = WasiUnexpect(detail::fromLastError(Error));
@@ -1207,9 +1210,9 @@ WasiExpect<void> INode::fdWrite(Span<Span<const uint8_t>> IOVs,
   NWritten = 0;
   for (size_t I = 0; I < Queries.size(); ++I) {
     auto &Query = Queries[I];
-    DWORD_ NumberOfBytesRead = 0;
+    DWORD_ NumberOfBytesWrite = 0;
     if (unlikely(
-            !GetOverlappedResult(Handle, &Query, &NumberOfBytesRead, true))) {
+            !GetOverlappedResult(Handle, &Query, &NumberOfBytesWrite, true))) {
       Result = WasiUnexpect(detail::fromLastError(GetLastError()));
       CancelIo(Handle);
       for (size_t J = I + 1; J < Queries.size(); ++J) {
@@ -1217,7 +1220,7 @@ WasiExpect<void> INode::fdWrite(Span<Span<const uint8_t>> IOVs,
       }
       break;
     }
-    NWritten += NumberOfBytesRead;
+    NWritten += NumberOfBytesWrite;
   }
 
   if (!Append) {
@@ -1393,7 +1396,7 @@ WasiExpect<void> INode::pathReadlink(std::string Path, Span<char> Buffer,
 
   // Fill the Buffer with the contents of the link
   HandleHolder Link(FullPath, FILE_GENERIC_READ_, FILE_SHARE_READ_,
-                    OPEN_EXISTING_, FILE_FLAG_OPEN_REPARSE_POINT_);
+                    OPEN_EXISTING_, 0);
 
   if (unlikely(!Link.ok())) {
     return WasiUnexpect(detail::fromLastError(GetLastError()));
